]> git.scottworley.com Git - overonion/commitdiff
Switch to stdio for output. It's simpler
authorScott Worley <scottworley@scottworley.com>
Sat, 6 May 2017 17:13:43 +0000 (10:13 -0700)
committerScott Worley <scottworley@scottworley.com>
Fri, 20 Oct 2017 08:26:41 +0000 (01:26 -0700)
reverse.c
reverse_lib.c
reverse_lib.h
reverse_test.c

index a9339073b1a48ffed53069222417a2ec3bc5d389..9bbf7b93076e4a61870f3f01c2e2010ed8ada58e 100644 (file)
--- a/reverse.c
+++ b/reverse.c
@@ -1,6 +1,7 @@
 #include "reverse_lib.h"
 
 #include <err.h>
+#include <stdio.h>
 #include <sysexits.h>
 
 int main(int argc, char** argv) {
@@ -8,7 +9,7 @@ int main(int argc, char** argv) {
     errx(EX_USAGE, "Usage: reverse filename");
   }
 
-  reverse_file(argv[1], 1);
+  reverse_file(argv[1], stdout);
 
   return 0;
 }
index dad7e0d733304ad1d7587dbc9dd006934c58bc27..46563562f3b3833d44a67b0809e5887c2abb949e 100644 (file)
@@ -1,26 +1,15 @@
 #define _FILE_OFFSET_BITS 64
 
-#define BUFFER_SIZE 4096
-
 #include <err.h>
 #include <fcntl.h>
+#include <stdio.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sysexits.h>
 #include <unistd.h>
 
-static void write_all(int fd, const void *buf, size_t count) {
-  const char* cbuf = buf;
-  size_t written = 0;
-  while (written < count) {
-    int ret = write(fd, &cbuf[written], count - written);
-    if (ret == -1) err(EX_IOERR, "Could not write");
-    written += ret;
-  }
-}
-
-void reverse_file(const char* input_filename, int output_fd) {
+void reverse_file(const char* input_filename, FILE* output_stream) {
   int fd = open(input_filename, O_RDONLY);
   if (fd == -1) err(EX_NOINPUT, "Could not open specified file");
 
@@ -33,17 +22,8 @@ void reverse_file(const char* input_filename, int output_fd) {
   char *m = mmap(NULL, map_size, PROT_READ, MAP_SHARED, fd, 0);
   if (m == MAP_FAILED) err(EX_NOINPUT, "Could not mmap input");
 
-  char buf[BUFFER_SIZE];
-  off_t buf_offset = 0;
   for (off_t p = stats.st_size - 1; p >= 0; p--) {
-    buf[buf_offset++] = m[p];
-    if (buf_offset >= BUFFER_SIZE) {
-      write_all(output_fd, buf, buf_offset);
-      buf_offset = 0;
-    }
-  }
-  if (buf_offset) {
-    write_all(output_fd, buf, buf_offset);
+    if (fputc(m[p], output_stream) == EOF) errx(EX_IOERR, "Could not write");
   }
 
   if (munmap(m, map_size) == -1) err(EX_IOERR, "Could not unmap input");
index f4d7dd677f91657708fe9a2abb25f2d9832e90ad..9c0f05ae2eaa9d75d1894ff4da8a0eb3c55eecfd 100644 (file)
@@ -1,8 +1,10 @@
 #ifndef _OVERONION_REVERSE_LIB_H
 #define _OVERONION_REVERSE_LIB_H
 
-/* Copy the contents of input_filename to output_fd backwards.
+#include <stdio.h>
+
+/* Copy the contents of input_filename to output_stream backwards.
  * input_filename must be a real file, not a pipe. */
-void reverse_file(const char* input_filename, int output_fd);
+void reverse_file(const char* input_filename, FILE* output_stream);
 
 #endif /* _OVERONION_REVERSE_LIB_H */
index fe35bcc303a04330c55e7bbd890f5fde8b590efb..512edaa7477b77d9d8a193bf40ca2b66ff985302 100644 (file)
@@ -13,8 +13,10 @@ char* reverse_to_temp_file(const char* input_file) {
   char* temp_filename = strdup("/tmp/reverse_test.XXXXXX");
   int fd = mkstemp(temp_filename);
   if (fd == -1) err(EX_IOERR, "Couldn't make a temporary file");
-  reverse_file(input_file, fd);
-  if (close(fd) == -1) err(EX_IOERR, "Couldn't close temporary file");
+  FILE* f = fdopen(fd, "w");
+  if (f == NULL) err(EX_IOERR, "Couldn't open temporary file");
+  reverse_file(input_file, f);
+  if (fclose(f) == EOF) err(EX_IOERR, "Couldn't close temporary file");
   return temp_filename;
 }